package Dialog;

#
# CREATE FUNCTION dialog(
#     title VARCHAR(32),
#     content TEXT,
#     options VARCHAR(255)
#  )
#  RETURNS VARCHAR(255)
#  LANGUAGE Perl
#  NO SQL
#  EXTERNAL NAME 'Dialog::dialog';
#

use 5.008008;
use strict;
use warnings;
use Symbol qw(delete_package);

# Tk is not thread safe, the native xs routines make a few assumptions
# and don't tidy up properly so if the Tk module is loaded in the dialog
# routine, it will crash somewhere in tkUnixXft.c

use Tk;
require Tk::Dialog;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(  );
our @EXPORT = qw( dialog );
our $VERSION = '0.01';

# Preloaded methods go here.

our $cw = undef;

sub empty_frame($)
{
  my ($cw) = @_;
  eval {
    while (my $subwidget = shift @{$cw->children}) {
      $subwidget->destroy;
    }
  }
}


sub setup_window
{
  my $cw = MainWindow->new;
  $cw->withdraw;
  my $top = $cw->Component('Frame', 'top');
  $top->configure(qw/-relief raised -bd 1/) unless $Tk::platform eq 'MSWin32';
  my $bot = $cw->Component('Frame', 'bottom');
  $bot->configure(qw/-relief raised -bd 1/) unless $Tk::platform eq 'MSWin32';
  $bot->pack(qw/-side bottom -fill both -ipady 3 -ipadx 3/);
  $top->pack(qw/-side top -fill both -ipady 3 -ipadx 3 -expand 1/);
}

sub dialog($$$)
{
  my ($title,$content,$options) = @_;
  my $selected_button = undef;

  {
    #require Tk;
    #require Tk::MainWindow;

    my @buttons = ();
    my $default_button = undef;
    $cw = MainWindow->new if !defined $cw;
    
    $cw->configure(
      -title => $title,
    );

    @buttons = split /,/, $options if defined($options) && ($options ne '');

    $cw->{'selected_button'} = undef;
    $cw->withdraw;

    @buttons = ( 'OK' ) if @buttons == 0;
    if (@buttons == 1) {
      $default_button = $buttons[0];
    }
    else {
      foreach my $bl (@buttons) {
        $default_button = $bl if $bl =~ m/^\<.*\>$/;
      }
    }

    my $top = $cw->Subwidget('top');
    $top = $cw->Component('Frame', 'top') if !defined $top;
    empty_frame($top);
    $top->configure(qw/-relief raised -bd 1/) unless $Tk::platform eq 'MSWin32';
    my $bot = $cw->Subwidget('bottom');
    $bot = $cw->Component('Frame', 'bottom') if !defined $bot;
    empty_frame($bot);
    $bot->configure(qw/-relief raised -bd 1/) unless $Tk::platform eq 'MSWin32';
    $bot->pack(qw/-side bottom -fill both -ipady 3 -ipadx 3/);
    $top->pack(qw/-side top -fill both -ipady 3 -ipadx 3 -expand 1/);

    foreach my $bl (@buttons) {
      my $text = $bl;
      $text = $1 if $bl =~ m/^\<(.*)\>$/;
      my $b = $bot->Button(
        -text => $text,
        -command => sub { $cw->{'selected_button'} = "$text" },
      );
      $b->bind('<Return>' => [ $b, 'Invoke' ]);
      $cw->Advertise("B_$bl" => $b);
      $b->configure(qw/-width 10 -pady 0/) if $Tk::platform eq 'MSWin32';
      if (defined($default_button) && ($bl eq $default_button)) {
        if ($Tk::platform eq 'MSWin32') {
          $b->pack(qw/-side left -expand 1 padx 1 -pady 1/);
        }
        else {
          my $db = $bot->Frame(qw/-relief sunken -bd 1/);
          $b->raise($db);
          $b->pack(
            -in => $db,
            -padx => 2,
            -pady =>2
          );
          $db->pack(qw/-side left -expand 1 -padx 1 -pady 1/);
        }
        $cw->{'default_button'} = $b;
        $cw->bind('<Return>' => [ $b, 'Invoke' ]);
      }
      else {
        $b->pack(qw/-side left -expand 1 -padx 1 -pady 1/);
      }
    }

    if (defined $default_button) {
      $cw->protocol('WM_DELETE_WINDOW' => sub { $cw->{'default_button'}->invoke })
    }
    else {
      $cw->protocol('WM_DELETE_WINDOW' => sub { $cw->{'selected_button'} = undef });
    }

    $cw->ConfigSpecs(
      -command     => ['CALLBACK', undef, undef, undef],
      -foreground  => ['DESCENDANTS', 'foreground', 'Foreground', 'black'],
      -background  => ['DESCENDANTS', 'background', 'Background', undef],
      -focus       => ['PASSIVE', undef, undef, undef],
      -showcommand => ['CALLBACK', undef, undef, undef],
    );
    $cw->Delegates('Construct', $top);

    my ($w_bitmap,$w_but,$pad1,$pad2);

    # Create the Toplevel window and divide it into top and bottom parts.

    my (@pl) = (-side => 'top', -fill => 'both');

    ($pad1, $pad2) =
        ([-padx => '3m', -pady => '3m'], [-padx => '3m', -pady => '2m']);

    $cw->iconname('Dialog');

    my $w_top = $cw->Subwidget('top');

    # Fill the top part with the bitmap and message.

    @pl = (-side => 'left');  

    $w_bitmap = $w_top->Label(Name => 'bitmap');
    $w_bitmap->pack(@pl, @$pad1);

    my $w_msg = $w_top->Label(
      -text       => $content,
      -wraplength => '3i',
      -justify    => 'left'
    );

    $w_msg->pack(-side => 'right', -expand => 1, -fill => 'both', @$pad1);

    $cw->Advertise(message => $w_msg);
    $cw->Advertise(bitmap  => $w_bitmap );

    $cw->ConfigSpecs(
      -image      => ['bitmap',undef,undef,undef],
      -bitmap     => ['bitmap',undef,undef,undef],
      -font       => ['message','font','Font', '-*-Times-Medium-R-Normal--*-180-*-*-*-*-*-*'],
      DEFAULT     => ['message',undef,undef,undef]
    );

    $cw->Popup;

    $cw->grab;
    if (defined $cw->{'default_button'}) {
      $cw->{'default_button'}->focus;
    }
    else {
      $cw->focus;
    }

    $cw->Callback(-showcommand => $cw);
    $cw->waitVariable(\$cw->{'selected_button'});
    $cw->grabRelease;
    $cw->withdraw;
    $cw->Callback(-command => $cw->{'selected_button'});

    $selected_button = $cw->{'selected_button'};

    #$cw->destroy;
  };
  
  return $selected_button;
}

sub olddialog($$$)
{
  my ($title,$content,$options) = @_;
  my $selected_button = undef;

  {
    #require Tk;
    #require Tk::MainWindow;

    my @buttons = ();
    my $default_button = undef;
    $cw = MainWindow->new if !defined $cw;
    
    $cw->configure(
      -title => $title,
    );

    @buttons = split /,/, $options if defined($options) && ($options ne '');

    $cw->{'selected_button'} = undef;
    $cw->withdraw;

    @buttons = ( 'OK' ) if @buttons == 0;
    if (@buttons == 1) {
      $default_button = $buttons[0];
    }
    else {
      foreach my $bl (@buttons) {
        $default_button = $bl if $bl =~ m/^\<.*\>$/;
      }
    }

    my $top = $cw->Subwidget('top');
    $top = $cw->Component('Frame', 'top') if !defined $top;
    empty_frame($top);
    $top->configure(qw/-relief raised -bd 1/) unless $Tk::platform eq 'MSWin32';
    my $bot = $cw->Subwidget('bottom');
    $bot = $cw->Component('Frame', 'bottom') if !defined $bot;
    empty_frame($bot);
    $bot->configure(qw/-relief raised -bd 1/) unless $Tk::platform eq 'MSWin32';
    $bot->pack(qw/-side bottom -fill both -ipady 3 -ipadx 3/);
    $top->pack(qw/-side top -fill both -ipady 3 -ipadx 3 -expand 1/);

    foreach my $bl (@buttons) {
      my $text = $bl;
      $text = $1 if $bl =~ m/^\<(.*)\>$/;
      my $b = $bot->Button(
        -text => $text,
        -command => sub { $cw->{'selected_button'} = "$text" },
      );
      $b->bind('<Return>' => [ $b, 'Invoke' ]);
      $cw->Advertise("B_$bl" => $b);
      $b->configure(qw/-width 10 -pady 0/) if $Tk::platform eq 'MSWin32';
      if (defined($default_button) && ($bl eq $default_button)) {
        if ($Tk::platform eq 'MSWin32') {
          $b->pack(qw/-side left -expand 1 padx 1 -pady 1/);
        }
        else {
          my $db = $bot->Frame(qw/-relief sunken -bd 1/);
          $b->raise($db);
          $b->pack(
            -in => $db,
            -padx => 2,
            -pady =>2
          );
          $db->pack(qw/-side left -expand 1 -padx 1 -pady 1/);
        }
        $cw->{'default_button'} = $b;
        $cw->bind('<Return>' => [ $b, 'Invoke' ]);
      }
      else {
        $b->pack(qw/-side left -expand 1 -padx 1 -pady 1/);
      }
    }

    if (defined $default_button) {
      $cw->protocol('WM_DELETE_WINDOW' => sub { $cw->{'default_button'}->invoke })
    }
    else {
      $cw->protocol('WM_DELETE_WINDOW' => sub { $cw->{'selected_button'} = undef });
    }

    $cw->ConfigSpecs(
      -command     => ['CALLBACK', undef, undef, undef],
      -foreground  => ['DESCENDANTS', 'foreground', 'Foreground', 'black'],
      -background  => ['DESCENDANTS', 'background', 'Background', undef],
      -focus       => ['PASSIVE', undef, undef, undef],
      -showcommand => ['CALLBACK', undef, undef, undef],
    );
    $cw->Delegates('Construct', $top);

    my ($w_bitmap,$w_but,$pad1,$pad2);

    # Create the Toplevel window and divide it into top and bottom parts.

    my (@pl) = (-side => 'top', -fill => 'both');

    ($pad1, $pad2) =
        ([-padx => '3m', -pady => '3m'], [-padx => '3m', -pady => '2m']);

    $cw->iconname('Dialog');

    my $w_top = $cw->Subwidget('top');

    # Fill the top part with the bitmap and message.

    @pl = (-side => 'left');  

    $w_bitmap = $w_top->Label(Name => 'bitmap');
    $w_bitmap->pack(@pl, @$pad1);

    my $w_msg = $w_top->Label(
      -text       => $content,
      -wraplength => '3i',
      -justify    => 'left'
    );

    $w_msg->pack(-side => 'right', -expand => 1, -fill => 'both', @$pad1);

    $cw->Advertise(message => $w_msg);
    $cw->Advertise(bitmap  => $w_bitmap );

    $cw->ConfigSpecs(
      -image      => ['bitmap',undef,undef,undef],
      -bitmap     => ['bitmap',undef,undef,undef],
      -font       => ['message','font','Font', '-*-Times-Medium-R-Normal--*-180-*-*-*-*-*-*'],
      DEFAULT     => ['message',undef,undef,undef]
    );

    $cw->Popup;

    $cw->grab;
    if (defined $cw->{'default_button'}) {
      $cw->{'default_button'}->focus;
    }
    else {
      $cw->focus;
    }

    $cw->Callback(-showcommand => $cw);
    $cw->waitVariable(\$cw->{'selected_button'});
    $cw->grabRelease;
    $cw->withdraw;
    $cw->Callback(-command => $cw->{'selected_button'});

    $selected_button = $cw->{'selected_button'};

    #$cw->destroy;
  };
  
  return $selected_button;
}

1;
__END__

