package Expert;

#
# CREATE PROCEDURE expert_evaluate(goal VARCHAR(255))
#   LANGUAGE Perl
#   MODIFIES SQL DATA
#   DYNAMIC RESULT SETS 1
#   EXTERNAL NAME 'Expert::evaluate'
#
# CREATE PROCEDURE expert_explain(goal VARCHAR(255))
#   LANGUAGE Perl
#   MODIFIES SQL DATA
#   DYNAMIC RESULT SETS 1
#   EXTERNAL NAME 'Expert::explain'
#

use 5.008008;
use strict;
use warnings;
use DBI;
use Dialog;
use Pie::KnowledgeBase;
use Pie::Engine;
use Pie::MySQL;
use Pie::Sequence;
use Symbol qw(delete_package);

require Exporter;

our @ISA = qw(Exporter);

our %EXPORT_TAGS = ( 'all' => [ qw(

) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(

);

our $VERSION = '0.01';

# Preloaded methods go here.

Pie::Sequence::set_type('MySQL');

sub test()
{
  my %result = ();
  $result{'Proc Demo'} = 'nifty test';
  return \%result;
}

sub init_kb
{
  my $kb = shift;
  	# Now add a couple of rules, and questions...

	my $r=$kb->new_rule("connection state");
	$r->add_condition("link.state","up");
	$r->add_condition("local.router.state","up");
	$r->add_condition("remote.router.state","up");
	$r->add_action("internet.connection","up");

	$r=$kb->new_rule("Link down");
	$r->add_condition("link.state","down");
	$r->add_action("internet.connection","down");

	$r=$kb->new_rule("local router");
	$r->add_condition("local.router.state","down");
	$r->add_action("internet.connection","down");

	$r=$kb->new_rule("remote router");
	$r->add_condition("remote.router.state","down");
	$r->add_action("internet.connection","down");

	$r=$kb->new_rule("DNS1");
	$r->add_condition("dns1.state","up");
	$r->add_action("dns.service","up");

	$r=$kb->new_rule("DNS2");
	$r->add_condition("dns2.state","up");
	$r->add_action("dns.service","up");

	$r=$kb->new_rule("DNS down");
	$r->add_condition("dns1.state","down");
	$r->add_condition("dns2.state","down");
	$r->add_action("dns.service","down");

	$r=$kb->new_rule("Internet state");
	$r->add_condition("dns.service","up");
	$r->add_condition("internet.connection","up");
	$r->add_action("internet.status", "up");

	$r=$kb->new_rule("Internet state nodns");
	$r->add_condition("dns.service","down");
	#$r->add_condition("internet.connection","up");
	$r->add_action("internet.status", "down");

	$r=$kb->new_rule("Internet state nonet");
	#$r->add_condition("dns.service","up");
	$r->add_condition("internet.connection","down");
	$r->add_action("internet.status", "down");

	$r=$kb->new_question("link.state","SELECT 'up' FROM DUAL");
	$r->add_response("up");
	$r->add_response("down");

	$r=$kb->new_question("local.router.state","SELECT 'up' FROM DUAL");
	$r->add_response("up");
	$r->add_response("down");

	$r=$kb->new_question("remote.router.state","SELECT 'up' FROM DUAL");
	$r->add_response("up");
	$r->add_response("down");

	$r=$kb->new_question("dns1.state","SELECT 'up' FROM DUAL");
	$r->add_response("up");
	$r->add_response("down");

	$r=$kb->new_question("dns2.state","SELECT 'up' FROM DUAL");
	$r->add_response("up");
	$r->add_response("down");
}


sub evaluate($)
{
  my ($goal) = @_;
  my %result = ();
  
  my $dbh = pie_db_connect('pie', undef, undef);
  
  eval {    
    my $kb = Pie::KnowledgeBase->new(
      'kb_type' => 'MySQL',
    );

    print "evaluate called\n";

    #init_kb($kb);

    my $engine = Pie::Engine->new($kb, (
      'attributes' => 'MySQL',
    ));
    $engine->mixed_mode(1);
    my $ret = $engine->back_chain($goal);
    if($ret eq "solved")
    {
      my $attr = $engine->get_attribute($goal);
      $result{'value'} = $attr->value;
    }
    $result{'attribute'} = $goal;
    $result{'status'} = $ret;
  };
  if ($@) {
    my $error = $@;
    pie_db_disconnect();
    die $error;
  }
  pie_db_disconnect();
  return \%result;
}

sub explain($)
{
  my ($goal) = @_;
  my @reasons = ();

  my $dbh = pie_db_connect('pie', undef, undef);
  
  eval {
    my $kb = Pie::KnowledgeBase->new(
      'kb_type' => 'MySQL',
    );
    my $engine = Pie::Engine->new($kb, (
      'attributes' => 'MySQL',
    ));
    $engine->mixed_mode(1);
    $engine->back_chain($goal);
    my @working = $engine->show_working($goal);
    foreach my $work (@working) {
      my $ref = ${$work}{'reasons'};
      my $reason;
      foreach $reason (@$ref) {
        my %row = ( 'rule' => ${$work}{'rule'},
                    'attribute' => $reason->name,
                    'value' => $reason->value,
                    'confidence' => $reason->confidence );
        push @reasons, \%row;
      }
    }
  };
  if ($@) {
    my $error = $@;
    pie_db_disconnect();
    die $error;
  }
  pie_db_disconnect();
  return \@reasons;
}

1;
__END__

